package videoautomat;
import java.awt.Rectangle;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;

import log.Log;
import sale.Action;
import sale.CalendarTime;
import sale.MenuSheet;
import sale.MenuSheetItem;
import sale.SaleProcess;
import sale.SalesPoint;
import sale.Shop;
import sale.StepTimer;
import users.UserManager;
import data.QuoteValue;
import data.Value;
import data.events.VetoException;
import data.ooimpl.CatalogImpl;
import data.ooimpl.CountingStockImpl;
import data.ooimpl.EUROCurrencyImpl;
import data.ooimpl.MoneyBagImpl;

/**
 * This is the central class for this application - the <code>Shop</code>. It contains the
 * video-catalog and stock of this video-shop, the shop`s money and therefor a currency-catalog
 * containing which coins this automat accepts and a <code>Timer</code> for simulation purposes.
 *  
 */
public class VideoShop extends Shop {
	/**
	 * Key of the video -<code>Catalog</code>
	 */
	public static final String C_VIDEOS = "VideoCatalog";
	/**
	 * Key of the currency -<code>Catalog</code>
	 */
	public static final String C_CURRENCY = "CurrencyCatalog";
	/**
	 * Key of the video -<code>CountingStock</code>
	 */
	public static final String CC_VIDEOS = "VideoStock";
	/**
	 * Key of the <code>MoneyBag</code>
	 */
	public static final String MB_MONEY = "Money";
	/**
	 * The name of the global log file
	 */
	public static final String FILENAME = "automat.log";

    /** The Caption of the <code>VideoAutomat</code> */
    public static final String CAPTION_AUTOMAT = "****** VIDEOAUTOMAT *** 24 H ******";    
    
    /** Label for the menusheet that contains self-defined items */
    public static final String MS_NEW = "Videoautomat";

    /** Label for the button to start the automat */
    public static final String MSI_AUTOMAT = "Start automat";
    
    /**
     * Label for the admin- <code>ActionCapability</code>, if it`s not granted.
     */
    public static final String MSG_ACCESS = "Acces denied!!!";     
    /**
     * Label for the time setter button
     */
    public static final String MSI_DAY = "+ 1 Day";
	/**
	 * Constructs a new VideoShop and set it as <code>Shop#setTheShop()</code>. Also initialize
	 * the global <code>Catalogs</code> and <code>Stocks</code> and sets a <code>Timer</code>.
	 *  
	 */
	public VideoShop() {
		super();
		setShopFrameBounds(new Rectangle(0, 0, 640, 480));

		addCatalog(new CatalogImpl(C_VIDEOS));
		addCatalog(new EUROCurrencyImpl(C_CURRENCY));
		addStock(
			new CountingStockImpl(
				CC_VIDEOS,
				(CatalogImpl) getCatalog(C_VIDEOS)));
		
		UserManager.setGlobalUM(new UserManager());
		
		addStock(
			new MoneyBagImpl(
				MB_MONEY,
				(EUROCurrencyImpl) getCatalog(C_CURRENCY)));
		
		setTimer(new StepTimer(new CalendarTime(System.currentTimeMillis())));
		setShopFrameTitle(getTimer().getTime().toString());
		try {
			Log.setGlobalOutputStream(new FileOutputStream(FILENAME, true));
		} catch (IOException ioex) {
			System.err.println("Unable to create log file.");
		}
	}

	/**
	 * @return the <code>Shop`s MenuSheet</code>, containing the default one, a button to start
	 *              an automat and a button to switch the time further.
	 * @see sale.Shop#createShopMenuSheet()
	 */
	protected MenuSheet createShopMenuSheet() {
		MenuSheet ms_default = super.createShopMenuSheet();
		MenuSheet ms_new = new MenuSheet(MS_NEW);
		MenuSheetItem msi_automat =
			new MenuSheetItem(MSI_AUTOMAT, new Action() {
			public void doAction(SaleProcess p, SalesPoint sp)
				throws Throwable {
				addSalesPoint(new VideoAutomat());
			}
		});
		ms_new.add(msi_automat);
		MenuSheetItem msi_time =
			new MenuSheetItem(MSI_DAY, new Action() {
			public void doAction(SaleProcess p, SalesPoint sp)
				throws Throwable {
				getTimer().goAhead();
				setShopFrameTitle(getTimer().getTime().toString());
			}
		});
		ms_new.add(msi_time);
		ms_default.add(ms_new);
		return ms_default;
	}
	/**
	 * Overidden to avoid the annoying save-query, when quiting the application.
	 * 
	 * @see sale.Shop#quit()
	 */
	public void quit() {
		if (shutdown(false)) {
			System.exit(0);
		}
	}
	/**
	 * Helper method to avoid to long code-lines.
	 * 
	 * @return the global <code>Catalog</code> of videos.
	 */
	public static CatalogImpl getVideoCatalog() {
		return (CatalogImpl) Shop.getTheShop().getCatalog(C_VIDEOS);
	}
	/**
	 * Helper method to avoid to long code-lines.
	 * 
	 * @return the global <code>Stock</code> of videos
	 */
	public static CountingStockImpl getVideoStock() {
		return (CountingStockImpl) Shop.getTheShop().getStock(CC_VIDEOS);
	}

	/**
	 * Helper method to avoid to long code-lines.
	 * 
	 * @return the global <code>MoneyBag</code> containing the money of the shop.
	 */
	public static MoneyBagImpl getMoneyBag() {
		return (MoneyBagImpl) Shop.getTheShop().getStock(MB_MONEY);
	}
	/**
	 * Helper method to avoid to long code-lines.
	 * 
	 * @return the global <code>EUROCurrencyImpl</code> -instance
	 */
	public static EUROCurrencyImpl getCurrency() {
		return (EUROCurrencyImpl) Shop.getTheShop().getCatalog(C_CURRENCY);
	}

	/**
	 * Method to iterate over all rented videos and taking out those, which renting costs exceed
	 * the purchase price
	 *  
	 */
	public static void checkRentedVideos() {
		Iterator it_users = UserManager.getGlobalUM().getUsers().iterator();
		while (it_users.hasNext()) {
			AutomatUser c = (AutomatUser) it_users.next();
			Iterator it_stock = c.getVideoStock().iterator(null, false);
			while (it_stock.hasNext()) {
				VideoCassette vc = (VideoCassette) it_stock.next();
				Value v =
					((QuoteValue) vc.getAssociatedItem(null).getValue())
						.getOffer();
				if (vc.getCost().compareTo(v) >= 0) {
					try {
						c.getVideoStock().remove(vc, null);
					} catch (VetoException e) {
						e.printStackTrace();
					}
				}
			}
		}
	}
}
